-include_lib("eNet/include/eNet.hrl").

-define(DefWsOpts, [
   binary
   , {packet, 4}
   , {active, false}
   , {reuseaddr, true}
   , {nodelay, false}
   , {delay_send, true}
   , {send_timeout, 15000}
   , {keepalive, true}
   , {exit_on_close, true}
   , {back_log, 1024}
]).

-export_type([wsOpt/0]).
-type wsOpt() ::
   listenOpt() |
   {wsMod, module()}.

-record(req, {
   method :: elli:http_method(),
   scheme :: undefined | binary(),
   host :: undefined | binary(),
   port :: undefined | 1..65535,
   path :: [binary()],
   args :: [{binary(), any()}],
   raw_path :: binary(),
   version :: wsHttp:version(),
   headers :: elli:headers(),
   original_headers :: elli:headers(),
   body :: elli:body(),
   pid :: pid(),
   socket :: undefined | wsNet:socket(),
   callback :: wsHer:callback()
}).




-define(EXAMPLE_CONF, [{callback, elli_example_callback}, {callback_args, []}]).


-export_type([callback/0, callback_mod/0, callback_args/0, event/0, result/0]).

%% @type callback(). A tuple of a {@type callback_mod()} and {@type
%% callback_args()}.
-type callback() :: {callback_mod(), callback_args()}.

%% @type callback_mod(). A callback module.
-type callback_mod() :: module().

%% @type callback_args(). Arguments to pass to a {@type callback_mod()}.
-type callback_args() :: list().

%% @type event(). Fired throughout processing a request.
%% See {@link elli_example_callback:handle_event/3} for descriptions.
-type event() ::
elli_startup|
bad_request    |
file_error|
chunk_complete |
request_complete|
request_throw  |
request_error       |
request_exit|
request_closed |
request_parse_error|
client_closed  |
client_timeout|
invalid_return.

-type result() ::
{elli:response_code() |ok, elli:headers(), {file, file:name_all()}|
{file, file:name_all(), wsUtil:range()}}|
{elli:response_code() | ok, elli:headers(), elli:body()}|
{elli:response_code() | ok, elli:body()}|
{chunk, elli:headers()}|
{chunk, elli:headers(), elli:body()}|
ignore.

-type sendfile_opts() :: [{chunk_size, non_neg_integer()}].